var D = Object.defineProperty;
var P = (o, e, t) => e in o ? D(o, e, { enumerable: !0, configurable: !0, writable: !0, value: t }) : o[e] = t;
var S = (o, e, t) => (P(o, typeof e != "symbol" ? e + "" : e, t), t);
class $ {
  /**
   * Constructor for the class
   * @param  {VueComponent} component
   */
  constructor(e, t) {
    this.ResizeSensor = e, this.breakpoints = {
      "min-width": [],
      "max-width": []
    }, this.init(t);
  }
  /**
   * Initialize ElementQuery by crawling stylesheets for breakpoints,
   * setting a resize watcher on the element, and firing the initial resize
   * event.
   * @param  {HTMLElement}  element
   * @return {void}
   */
  init(e) {
    this.setBreakpoints(e), this.watchElement(e), this.setInitialResize(e);
  }
  /**
   * Fire the initial resize event
   * @param {HTMLElement} element
   */
  setInitialResize(e) {
    e.resizedAttached.call();
  }
  /**
   * Create a watcher on the element width
   * @param {HTMLElement} element
   * @return {void}
   */
  watchElement(e) {
    new this.ResizeSensor(e, () => {
      e.setAttribute("min-width", this.setMinWidth(e.clientWidth)), e.setAttribute("max-width", this.setMaxWidth(e.clientWidth));
    });
  }
  /**
   * Crawl the document stylesheets and capture all breakpoints for
   * this element
   * @param {HTMLElement} element
   */
  setBreakpoints(e) {
    this.crawlStylesheets(document, (t) => {
      this.readRules(t, (i) => {
        this.matchesScope(e, i.selectorText) && this.breakpoints[i.selector].push(i.value);
      });
    });
  }
  /**
   * Determine if an element has a scoped stylesheet. This prevents
   * us from crawling stylesheet rules that aren't relevant.
   * @param  @param {HTMLElement} element
   * @param  {String} selectorText
   * @return {void}
   */
  matchesScope(e, t) {
    let i = this.getScopeTag(e);
    return t.match(i);
  }
  /**
   * Returns a regex to determine if an element is scoped
   * @param  @param {HTMLElement}  element
   * @param  {RegExp}  regex
   * @return {RegExp}
   */
  getScopeTag(e) {
    let t = Object.keys(e.dataset).find((i) => i.match(/v-/));
    if (!t)
      throw new Error({
        message: "[VueElementQuery] Vue element queries only work on scoped stylesheets. Did you forget to add `scoped` to your stylesheet?"
      });
    return new RegExp(t);
  }
  /**
   * Set the string for the min-width attribute that will be used for element
   * queries. Example result: "500px 600px 900px"
   * @param {Number} elementWidth
   */
  setMinWidth(e) {
    return this.breakpoints["min-width"].reduce((t, i) => e >= i ? t + i + "px " : t, "").slice(0, -1);
  }
  /**
   * Set the string for the max-width attribute that will be used for element
   * queries. Example result: "500px 600px 900px"
   * @param {Number} elementWidth
   */
  setMaxWidth(e) {
    return this.breakpoints["max-width"].reduce((t, i) => e <= i ? t + i + "px " : t, "").slice(0, -1);
  }
  /**
   * Crawl the document for stylesheets, and if valid for reading rules,
   * pass to a callback function with the rules as the first argument.
   * @param  {HTMLDocument}  document
   * @param  {Function}  callback
   * @return {void}
   */
  crawlStylesheets(e, t) {
    for (var i = 0, d = e.styleSheets.length; i < d; i++)
      this.ifValidRules(e.styleSheets[i], t);
  }
  /**
   * Determine if the rules are valid for crawling on a stylesheet.
   * @param  {CSSStyleSheet}  stylesheet
   * @param  {Function}  callback
   * @return {void}
   */
  ifValidRules(e, t) {
    e.cssRules && t(e.cssRules);
  }
  /**
   * Read from the list of rules and build a map
   * @param  {CSSRuleList}  rules
   * @return {[type]}
   */
  readRules(e, t) {
    this.crawlRules(e, (i) => {
      t(i);
    });
  }
  /**
   * Crawl rules and execute a callback if the rule has valid
   * selectorText.
   * @param  {CSSRuleList}  rules
   * @param  {Function}  callback
   * @return {void}
   */
  crawlRules(e, t) {
    for (var i = 0, d = e.length; i < d; i++)
      e[i].selectorText && this.getValidSelector(e[i].selectorText, t);
  }
  /**
   * Check for a valid selector
   * @param  {String}  selectorText
   * @param  {Function} callback
   * @return {void}
   */
  getValidSelector(e, t) {
    ["min-width", "max-width"].forEach((d) => {
      if (e.indexOf(d) !== -1) {
        let c = new RegExp(`${d}~="(.*)px"]`);
        const g = e.match(c);
        g && g.length >= 2 && t({
          selectorText: e,
          selector: d,
          value: g[1]
        });
      }
    });
  }
}
var G = typeof globalThis < "u" ? globalThis : typeof window < "u" ? window : typeof global < "u" ? global : typeof self < "u" ? self : {};
function U(o) {
  return o && o.__esModule && Object.prototype.hasOwnProperty.call(o, "default") ? o.default : o;
}
var V = { exports: {} };
(function(o, e) {
  (function(t, i) {
    o.exports = i();
  })(typeof window < "u" ? window : G, function() {
    if (typeof window > "u")
      return null;
    var t = typeof window < "u" && window.Math == Math ? window : typeof self < "u" && self.Math == Math ? self : Function("return this")(), i = t.requestAnimationFrame || t.mozRequestAnimationFrame || t.webkitRequestAnimationFrame || function(r) {
      return t.setTimeout(r, 20);
    }, d = t.cancelAnimationFrame || t.mozCancelAnimationFrame || t.webkitCancelAnimationFrame || function(r) {
      t.clearTimeout(r);
    };
    function c(r, a) {
      var n = Object.prototype.toString.call(r), l = n === "[object Array]" || n === "[object NodeList]" || n === "[object HTMLCollection]" || n === "[object Object]" || typeof jQuery < "u" && r instanceof jQuery || typeof Elements < "u" && r instanceof Elements, x = 0, s = r.length;
      if (l)
        for (; x < s; x++)
          a(r[x]);
      else
        a(r);
    }
    function g(r) {
      if (!r.getBoundingClientRect)
        return {
          width: r.offsetWidth,
          height: r.offsetHeight
        };
      var a = r.getBoundingClientRect();
      return {
        width: Math.round(a.width),
        height: Math.round(a.height)
      };
    }
    function z(r, a) {
      Object.keys(a).forEach(function(n) {
        r.style[n] = a[n];
      });
    }
    var m = function(r, a) {
      var n = 0;
      function l() {
        var s = [];
        this.add = function(v) {
          s.push(v);
        };
        var h, p;
        this.call = function(v) {
          for (h = 0, p = s.length; h < p; h++)
            s[h].call(this, v);
        }, this.remove = function(v) {
          var u = [];
          for (h = 0, p = s.length; h < p; h++)
            s[h] !== v && u.push(s[h]);
          s = u;
        }, this.length = function() {
          return s.length;
        };
      }
      function x(s, h) {
        if (s) {
          if (s.resizedAttached) {
            s.resizedAttached.add(h);
            return;
          }
          s.resizedAttached = new l(), s.resizedAttached.add(h), s.resizeSensor = document.createElement("div"), s.resizeSensor.dir = "ltr", s.resizeSensor.className = "resize-sensor";
          var p = {
            pointerEvents: "none",
            position: "absolute",
            left: "0px",
            top: "0px",
            right: "0px",
            bottom: "0px",
            overflow: "hidden",
            zIndex: "-1",
            visibility: "hidden",
            maxWidth: "100%"
          }, v = {
            position: "absolute",
            left: "0px",
            top: "0px",
            transition: "0s"
          };
          z(s.resizeSensor, p);
          var u = document.createElement("div");
          u.className = "resize-sensor-expand", z(u, p);
          var E = document.createElement("div");
          z(E, v), u.appendChild(E);
          var w = document.createElement("div");
          w.className = "resize-sensor-shrink", z(w, p);
          var C = document.createElement("div");
          z(C, v), z(C, { width: "200%", height: "200%" }), w.appendChild(C), s.resizeSensor.appendChild(u), s.resizeSensor.appendChild(w), s.appendChild(s.resizeSensor);
          var k = window.getComputedStyle(s), R = k ? k.getPropertyValue("position") : null;
          R !== "absolute" && R !== "relative" && R !== "fixed" && R !== "sticky" && (s.style.position = "relative");
          var M = !1, O = 0, y = g(s), j = 0, F = 0, W = !0;
          n = 0;
          var I = function() {
            var f = s.offsetWidth, b = s.offsetHeight;
            E.style.width = f + 10 + "px", E.style.height = b + 10 + "px", u.scrollLeft = f + 10, u.scrollTop = b + 10, w.scrollLeft = f + 10, w.scrollTop = b + 10;
          }, A = function() {
            if (W) {
              var f = s.offsetWidth === 0 && s.offsetHeight === 0;
              if (f) {
                n || (n = i(function() {
                  n = 0, A();
                }));
                return;
              } else
                W = !1;
            }
            I();
          };
          s.resizeSensor.resetSensor = A;
          var N = function() {
            O = 0, M && (j = y.width, F = y.height, s.resizedAttached && s.resizedAttached.call(y));
          }, q = function() {
            y = g(s), M = y.width !== j || y.height !== F, M && !O && (O = i(N)), A();
          }, L = function(f, b, H) {
            f.attachEvent ? f.attachEvent("on" + b, H) : f.addEventListener(b, H);
          };
          L(u, "scroll", q), L(w, "scroll", q), n = i(function() {
            n = 0, A();
          });
        }
      }
      c(r, function(s) {
        x(s, a);
      }), this.detach = function(s) {
        n || (d(n), n = 0), m.detach(r, s);
      }, this.reset = function() {
        r.resizeSensor.resetSensor();
      };
    };
    if (m.reset = function(r) {
      c(r, function(a) {
        a.resizeSensor.resetSensor();
      });
    }, m.detach = function(r, a) {
      c(r, function(n) {
        n && (n.resizedAttached && typeof a == "function" && (n.resizedAttached.remove(a), n.resizedAttached.length()) || n.resizeSensor && (n.contains(n.resizeSensor) && n.removeChild(n.resizeSensor), delete n.resizeSensor, delete n.resizedAttached));
      });
    }, typeof MutationObserver < "u") {
      var B = new MutationObserver(function(r) {
        for (var a in r)
          if (r.hasOwnProperty(a))
            for (var n = r[a].addedNodes, l = 0; l < n.length; l++)
              n[l].resizeSensor && m.reset(n[l]);
      });
      document.addEventListener("DOMContentLoaded", function(r) {
        B.observe(document.body, {
          childList: !0,
          subtree: !0
        });
      });
    }
    return m;
  });
})(V);
var J = V.exports;
const K = /* @__PURE__ */ U(J);
function X(o, e) {
  let t;
  return function() {
    clearTimeout(t);
    const i = arguments;
    t = setTimeout(() => {
      o.apply(this, i);
    }, e);
  };
}
class Y {
  constructor(e, t) {
    S(this, "resizeObserver", null);
    S(this, "element");
    S(this, "width", 0);
    S(this, "height", 0);
    S(this, "options");
    let i = e;
    typeof e == "string" && (i = document.querySelector(e)), this.element = i, this.options = Object.assign({
      delay: 150
    }, t), this.observe(this.element);
  }
  observe(e) {
    if (e ? this.element = e : e = this.element, !(e instanceof HTMLElement))
      throw new Error("The target element must be a HTMLElement");
    const { width: t, height: i } = e.getBoundingClientRect();
    this.width = Math.floor(t), this.height = Math.floor(i), typeof this.options.resize == "function" && this.options.immediate && this.options.resize({ width: t, height: i }, e), this.resizeObserver && this.disconnect(), this.resizeObserver = new ResizeObserver(this._onResize()), this.resizeObserver.observe(e);
  }
  _onResize() {
    const e = this.options.delay || this.options.wait;
    return e ? X(this._handleResize.bind(this), e) : this._handleResize.bind(this);
  }
  _handleResize(e) {
    for (const t of e) {
      const i = t.target;
      let { width: d, height: c } = i.getBoundingClientRect();
      d = Math.floor(d), c = Math.floor(c), (this.width !== d || this.height !== c) && (this.width = d, this.height = c, typeof this.options.resize == "function" && this.options.resize({ width: d, height: c }, i));
    }
  }
  disconnect() {
    this.resizeObserver && (this.resizeObserver.disconnect(), this.resizeObserver = null);
  }
  destroy() {
    this.disconnect(), this.element = null;
  }
}
const Q = {
  inserted(o, e) {
    const t = new $(Y, o);
    o.__vue_elementQuery__ = t;
  },
  unbind(o) {
    K.detach(o);
    const e = o.__vue_elementQuery__;
    e && e.destroy();
  }
}, T = {
  mounted: Q.inserted,
  beforeUnmount: Q.unbind
}, _ = (o, e = {}) => {
  _.installed || (_.installed = !0, o.directive(e.directive || "query", T));
}, ee = {
  install: _,
  // ResizeObserver,
  queryDirective: T,
  directive: T
};
export {
  ee as default
};
